--LoraWAN json, payload parser
--params:
-- SrcID (строка) - SRC_ID, если указан, то принимаются payload с указанным SRC_ID,
--         иначе принимаются все payload
-- szSrcID (целое число) - размер SRC_ID от 0 до 8, по умолчанию 8
-- szParamID (целое число) - размер PARAM_ID 1 или 2, по умолчанию 1
-- префиксы идентификаторов сигналов:
-- ts - дискретные (ТС) ts (ts0 .. ts127 )
-- ti - аналоговые (ТИ) ti (ti0 .. ti255 )
-- pid - параметры из json пакета param_id (pid3 - время, pid65 - батарея, pid63 - температура )

--init tm_sever functions
local log = tmsLog() --log file
local dt = tmsDate() -- date
local tms = tmsSigs() -- get and set sigs
local raw = tmsRawData() -- get and set raw datas
local pars = tms.pars() -- get params
--create tm_server objects
local json = tmsNewJsonReader() -- json parser
--declare global varables
local curDT
local SrcID
local szSrcID = 8
local szParamID = 1

function tmsStart() -- start function
  if pars.szSrcID ~= nil then
    if pars.szSrcID >= 0 and pars.szSrcID <= 8 then
      szSrcID = pars.szSrcID
    else
      log.ERR( "invalid param szSrcID ", pars.szSrcID )
    end
  end
  if pars.szParamID ~= nil then
    if pars.szParamID >= 1 and pars.szParamID <= 2 then
      szParamID = pars.szParamID
    else
      log.ERR( "invalid param szParamID ", pars.szParamID )
    end
  end  
  if pars.SrcID ~= nil then
    if (string.len( pars.SrcID ) / 2) == szSrcID then
      SrcID = pars.SrcID
    else
      log.ERR( "invalid size param SrcID ", pars.SrcID )
    end
  end
  log.MES( "SrcID:", SrcID, ", szSrcID:", szSrcID, ", szParamID:", szParamID )
end --tmsStart

--эту фунцию tm_server вызывает с указанным периодом в параметре workper
function tmsWork() -- main function
--debug parseJson( '[{"dev_eui":"16c000001011e4","param_id":-1,"link_id":72461633,"seq_number":0,"port_number":1,"data_timestamp":1633444827,"data_timestamp_time":"05-10-2021 14:40:27","value":"010203040506070802150A06100B22000F8102830405060708090A","description":"Raw data"},{"dev_eui":"16c000001011e4","param_id":-1,"link_id":72461633,"seq_number":0,"port_number":1,"data_timestamp":1633444827,"data_timestamp_time":"05-10-2021 14:40:27","value":"010203040506070803150a05112800011d150000002c160000002d17FFFFFFF2","description":"Raw data"},{"dev_eui":"16c000001011e4","param_id":3,"link_id":72461633,"seq_number":0,"port_number":0,"data_timestamp":1633444827,"data_timestamp_time":"05-10-2021 14:40:27","value":26793,"description":"Время работы трансивера в микросекундах "},{"dev_eui":"16c000001011e4","param_id":4,"link_id":72461633,"seq_number":0,"port_number":0,"data_timestamp":1633444827,"data_timestamp_time":"05-10-2021 14:40:27","value":65,"description":"Уровень заряда батареи 0 – 255,"},{"dev_eui":"16c000001011e4","param_id":63,"link_id":72461633,"seq_number":0,"port_number":0,"data_timestamp":1633444827,"data_timestamp_time":"05-10-2021 14:40:27","value":37,"description":"Температура"}]')
  local RD = raw.rawRead() -- чтение массива новых raw data {id1:val1,id2:val2,...}
  if RD == nil then
    return --нет новых
  end
  for id,val in pairs( RD ) do
    log.MES( "--> ", val.var )
    parseJson( val.data )
  end
end --tmsWork

function parseJson( s )
  log.MES( "--> ", s )
  curDT = dt.curDate()
  json:jsonOpen( s ) --строку в json parser
--чтение массива объектов [{"p1":"v1","p1":"v1"},{..},..]  
  json:beginArray() --[
  while json:hasNext() do --,
--чтение объекта  
    json:beginObject() --{
    local par, val, desc, pid, date = curDT
    while json:hasNext() do --,
      par = json:nextParam() --"PARAM":
      if par == 'value' then
        val = json:nextValue() --:"VALUE"
--        log.DEBUG(par,": ",val)
      elseif par == 'description' then
        desc = json:nextValue()
--        log.DEBUG(par,": ",desc)
      elseif par == 'param_id' then
        pid = tonumber( json:nextValue() )
--        log.DEBUG(par,": ",pid)
      elseif par == 'data_timestamp' then
        date = tonumber( json:nextValue() )
        date = date * 1000 --ms
--        log.DEBUG(par,": ",date)
      else
        json:skipValue()
      end
    end
    json:endObject() --}
-- объект прочитан - выполнить обработку    
    log.DEBUG(pid,": ",val," - ", desc)
    if pid == -1 then --raw data
      raw.rawWrite( "loraOut", val )
      parseRawData( val )
    else --параметры
      sigSend( 'pid'..pid, val, date )
    end
  end
  json:endArray() --]
  json:jsonClose() --удалить строку из json parser
end --parseJson


function sigSend( name, val, date ) --передача сигнала
  local ods = {}
  ods.VL = tonumber( val ) -- значение
  ods.ST = 0 --статус
  ods.DT = date --дата
  tms.set( name, ods ) -- to tm_server
  log.MES( "<== '"..name.."' VL:",ods.VL,",ST:",ods.ST,",DT:",ods.DT )
end --sigSend


function parseRawData( s )
--ts s = '010203040506070802150A06100B22000F8102830405060708090A' 
--ti s = '010203040506070803150a05112800011d150000002c160000002d17FFFFFFF2'
--ts s = 'AABB00A71180028003A714'
  if SrcID ~= nil and szSrcID ~= 0 then
    local curSrcID = string.sub( s, 1, szSrcID*2 )
    if curSrcID ~= SrcID then
      log.DEBUG( 'SRC_ID ', curSrcID,  ' skip' )
      return
    end
    log.DEBUG( 'SRC_ID ', curSrcID )
  end
  log.DEBUG( "parse raw data:\n",s)
  local pos = szSrcID + 1 --9
  local function posToIndex()
    return 2*pos-1
  end
  local function getByte()
    local i = posToIndex( pos ) 
    local svl = string.sub(s,i,i+1)
    local vl = tonumber( svl, 16 )
    log.DEBUG( "byte[",pos,"]=", vl, " <<< ", svl )
    pos = pos + 1
    return vl
  end
  local function getWord()
    local i = posToIndex() 
    local svl = string.sub(s,i,i+3)
    local vl = tonumber( svl, 16 )
    log.DEBUG( "word[", pos,"]=", vl, " <<< ", svl )
    pos = pos + 2
    return vl
  end
  local function getINT()
    local i = posToIndex() 
    local svl = string.sub(s,i,i+7) --unsigned
    local vl = tonumber( svl, 16 ) 
    if vl > 2147483647 then --signed
      vl = 4294967296 - vl
      vl = -vl
    end
    log.DEBUG( "INT [", pos,"]=", vl, " <<< ", svl )
    pos = pos + 4
    return vl
  end
  local function testBit( vl, bitPos )
    return (vl&(1<<bitPos)) ~= 0
  end
--0102030405060708 03 150a05112800011d 150000002c 160000002d 17FFFFFFF2
--0102030405060708 02 150A06100B22000F 0102030405060708090A5798
  local FLAGS = getByte()
  local analog = testBit( FLAGS, 0 )
  local withTime = testBit( FLAGS, 1 )
  log.INFO( "analog=", analog, " withTime=", withTime )
  local DT
  if withTime then
    local Y = getByte() + 2000
    local M = getByte()
    local D = getByte()
    local H = getByte()
    local mn = getByte()
    local S = getByte()
    local ms = getWord()
    DT = dt.build(Y,M,D,H,mn,S,ms) --LOC
    DT = dt.UTC( DT ) --UTC
    log.DEBUG( "date LOC: ",dt.frm(dt.LOC(DT)) ) -- формат по умолчанию
    log.INFO( "date UTC: ",dt.frm(DT, "%Y.%m.%d %H:%M:%S.%V") )
  else
    DT = curDT --UTC
  end
  local slen = string.len( s ) / 2
  local mes = ''
  local c = 0
  while pos <= slen do
    c = c + 1
    local PARAM_ID, VL, ID
    if szParamID == 1 then
      PARAM_ID = getByte()
    else
      PARAM_ID = getWord()
    end
    if not analog then
      if szParamID == 1 then
        ID = PARAM_ID & 0x7f--0x3f
        VL = PARAM_ID & 0x80
      else
        ID = PARAM_ID & 0x3fff
        VL = PARAM_ID & 0x8000
      end
      if VL ~= 0 then
        VL = 1
      end
      sigSend( 'ts'..ID, VL, DT )
    else
      ID = PARAM_ID
      VL = getINT()
      sigSend( 'ti'..ID, VL, DT )
    end --analog
--    sigSend( 'id'..ID, VL, DT )
    mes = mes.." "..ID..":"..VL
  end --while
  if not analog then
    mes = "discrete:"..mes
  else
    mes = "analog:"..mes
  end
  mes = "--- "..c.." "..mes.." ---"
  log.INFO( mes )
end --parseRawData
