-- Params to DB
--[[
Сохраняет сигналы (числа) и Rawdata (строки) в БД sqlite:
{"PAR1":1,"PAR2":2,"PAR3":4.33,"str1":"text2"}
Читает сигналы из БД после старта.
Данные из БД приоритетные.
параметры:
- filename (str) - полный путь и имя файла БД (пример: /tmp/db/pars.sqlite)
]]--
local log = tmsLog() --log file
local dt = tmsDate() -- date
local tm = tmsTick() -- timer
local tms = tmsSigs() -- get and set sigs
local raw = tmsRawData() -- get and set raw datas
local pars = tms.pars() -- get params
local db = tmsNewSQLiteDB() --SQLite БД
local sel = db:dbNewSelect() --для SELECT запросов к БД
local dbErrs = db.dbErrors() --константы кодов SQLite ошибок 
local attrs = tms.attrs() --константы атрибутов сигналов

function tmsWork() --периодически (см. параметр скрипта "Период обновления")
  updSig()
  updRaw()
end
function updSig()
--прием обновлений входных сигналов
  local ids = tms.news()
  if ids == nil then
    return--нет новых
  end
  db:dbOpen( pars.filename )
  db:dbExec( 'BEGIN' )
  for k,v in pairs(ids) do
    local DS = tms.get( v )
    if DS ~= nil and not isBad( DS ) then -- сохранять и передавать только достоверные сигналы 
      log.DEBUG( function() return v," ",DS.VL," ",DS.ST," ",dt.frm( DS.DT ) end )
      tms.set( v, DS )
      updateValue( v, DS.VL )
    end--if
  end--for
  db:dbExec( 'END' )
  db:dbClose()
end--updSig

function updRaw()
  local RD = raw.rawRead() -- чтение массива новых raw data {id1:val1,id2:val2,...}
  if RD == nil then
    return --нет новых
  end
  db:dbOpen( pars.filename )
  db:dbExec( 'BEGIN' )
  for _,rvl in pairs( RD ) do
    log.DEBUG( rvl.var, " ", rvl.data )
    raw.rawWrite( rvl.var, rvl.data )
    updateValue( rvl.var, rvl.data, true )
  end
  db:dbExec( 'END' )
  db:dbClose()
end--updRaw

function updateValue( id, vl, isStr )
  local vlOld
  sel:selOpen( 'SELECT * FROM tPAR WHERE fID="'..id..'"' )
  if not sel:selEnd() then
    vlOld = sel:selVal( 'fVL' ) -- сохраненное значение
  end
  sel:selClose()
  if vl ~= vlOld then -- обновить значение
    local s = 'DELETE FROM tPAR WHERE fID="'..id..'"' 
    log.TRACE( s )
    db:dbExec( s )
    if isStr then
      vl = '"'..vl..'"'
    end
    s = 'INSERT INTO tPAR VALUES("'..id..'",'..vl..')'
    log.TRACE( s )
    db:dbExec( s )
  end--if
end

function tmsStart() --после старта
  db:dbOpen( pars.filename )
  db:dbExec( 'PRAGMA journal_mode = DELETE' )
  db:dbExec( 'PRAGMA synchronous = 2' )
  sel:errCatch( false ) --не ловить ошибки 
  local s
  for i = 1,2 do
    s = 'SELECT * FROM tPAR'
    log.TRACE( s )
    sel:selOpen( s )
    local eNum, eStr = sel:errGet() --получить ошибку
    if eNum == dbErrs.SQLITE_OK then
      break --ошибок нет
    end
    --создать таблицу 
    log.WAR( eNum, ': ', eStr )
    s = 'CREATE TABLE tPAR ( fID VARCHAR NOT NULL, fVL VARCHAR NOT NULL, PRIMARY KEY( fID ) )'
    log.TRACE( s )
    db:dbExec( s )
  end
  sel:errCatch( true ) --ловить ошибки
  local curdt = dt.curDate()
  local id, vl
  while not sel:selEnd() do
    id = sel:selVal( 'fID' )
    vl = sel:selVal( 'fVL' )
    log.DEBUG( function() return id..' = '..vl end )--DEBUG
    local ds = tms.get( id )
    if ds ~= nil then
      sigSend( id, vl, curdt )
    else
      raw.rawWrite( id, vl )
      log.TRACE( function() return "<== '"..id.."' :", vl end )
    end
    sel:selNext()
  end--while
  sel:selClose()
  db:dbClose()
end --tmsStart
function tmsStop() --перед остановкой
  updSig()
  updRaw()
end --tmsStop

function sigSend( name, val, date ) --передача сигнала
  local ods = {}
  ods.VL = tonumber( val ) -- значение
  ods.ST = 0 --статус
  ods.DT = date --дата
  sigSendDS( name, ods )
end --sigSend
function sigSendDS( name, ods ) --передача сигнала
  tms.set( name, ods ) -- to tm_server
  log.TRACE( "<== '"..name.."' VL:",ods.VL,",ST:",ods.ST,",DT:",ods.DT )
end --sigSend

function isBad( ds )
  if ds.ST&attrs.aBAD ~= 0 then
    return true
  end
  return false
end


