-- краткая справка
--=================================================================
--Логика работы lua-скрипта и tm-сервера.
--После запуска tm-сервера lua-скрипт загружается и выполняются все глобальные объявления.
--Пример:
--объявления глобальных переменных
local log = tmsLog() --log file
local dt = tmsDate() -- date
local tms = tmsSigs() -- get and set sigs
local raw = tmsRawData() -- get and set raw datas
local utils = tmsUtils() -- utils functions
local pars = tms.pars() -- get params
local ret = tms.parseRet() -- parse ret codes 
--переменные состояния
local VLs = {} --таблица входных сигналов { w1=value1, w2=value2 ... }
local needWrite = false --true - необходимо выполнить запись регистров
local waitRes = 0 


--Далее tm-сервер вызывает функции:

function tmsWork() --периодически (см. параметр скрипта "Период обновления")
end
function tmsStart() --только после старта
end
function tmsStop() --только перед остановкой
end
function tmsParse( s )
-- Только, если lua-скрипт добавлен в компонент "RAW устройство".
-- Вызывается "RAW устройством" для обработки ответов и формирования запросов.
-- s - принятые данные от "RAW устройства" ввиде строки (строка может быть пустая).
-- вернуть (return) необходимо данные для передачи RAW устройству ввиде строки
-- или код возврата:
 ret.ERROR -- ошибка
 ret.MORE -- продолжать прием
 ret.NEXT -- перейти к следующему устройству.
-- Строка в lua может содержать двоичные данные (не текст).
-- функция string.unpack() - извлекает данные из двоичного буфера.
-- функция string.pack() - помещает данные в двоичный буфер.
-- Примеры https://fingercomp.gitbooks.io/oc-cookbook/content/lua/data-packing.html
end

--Ошибки lua-скрипта tm-сервер пишет в лог этого lua-скрипта.
--Отладку выполнять трассировкой значений в лог.

--=================================================================
--Функции
-------------------------------------------------------------------
tmsLog -- функция, возвращает таблицу функций для записи в лог-файл:
ERR -- ошибка 
WAR -- предупреждения
MES -- сообщения
INFO -- информация
DEBUG -- отладка
--Аргументы функций:
-- строки или значения, которые можно преобразовать в строку
-- или одна функция
--Примеры:
local log = tmsLog()
local vl1, vl2 = 10, 20
log.MES( 'значение1=', vl1, ' значение2', vl2 ) --значение1=10, значение2=20

--Если аргументов много и/или они сложно вычисляются, то лучше передавать 
--функцию для меньшей загрузки процессора, т.к. когда лог отключен
--параметры не передаются и не вычисляются.

log.DEBUG( function() return 'значение1=', vl1, ', crc=', calcCRC(s) end )

-------------------------------------------------------------------
tmsDate -- функция, возвращает таблицу функций для работы с датами.
--Дата int64 - количество миллисекунд прошедших с 01.01.1970.
local dt = tmsDate()

curDate -- возвращает текущую дату UTC
local cur = dt.curDate()

-- функции извлечения года, месяца и т.д. :
year, mon, day, hour, min, sec, mSec
, wDay --(день недели 1 - понедельник, 7 - воскресенье )
local Y = dt.year( dt.curDate() )

-- функции пересчета дней часов и т.д. в миллисекунды
msDay, msHour, msMin, msSec
local oldDT = dt.curDate() - dt.msDay( 7 ) -- минус 7 дней

build -- возвращает дату из года, месяца и т.д.
dt.build( year, mon, day, hour, min, sec, mSec )
dt.build( 2022, 1, 10 ) --2022.01.10 00:00:00.000
dt.build( 2022, 1, 10, 10, 30 ) --2022.01.10 10:30:00.000

part -- возвращает таблицу (структуру) с полями year, mon, day, hour, min, sec, msec, wday
local dtp = dt.part( dt.curDate() )
log.INFO( dtp.year,'.',dtp.mon,'.',dtp.day,'(',dtp.wday,') ',dtp.hour,':',dtp.min,':',dtp.sec,'.',dtp.msec )
 
frm -- возвращает строковое представление даты.
--параметры: дата, формат (поумолчанию "%Y.%m.%d %H:%M:%S.%V")
log.MES( dt.frm( dt.curDate() ) )
log.MES( dt.frm( dt.curDate(), "%Y-%m-%d %H:%M:%S" ) )

UTC -- локальную дату в UTC
local utcDT = dt.UTC( dt.build( 2022, 1, 10, 10, 30 ) )

LOC -- дату UTC в локальную 
local locDT = dt.LOC( dt.build( 2022, 1, 10, 10, 30 ) )

leep -- возвращает truе, если год высокосный
if dt.leep( dt.curDate() ) then
  --год высокосный
end
-------------------------------------------------------------------
tmsTick -- функция, возвращает таблицу функций для работы с таймером.
--Системный таймер - счетчик тиков с момента загрузки устройства.
--Вес тика в миллисекундах зависит от устройства.
local tm = tmsTick()

curTick -- функция, возвращает текущее значение таймера
local t = tm.curTick()

toMs -- функция, возвращает миллисекунды из тиков таймера. 
local ms = tm.toMs( t )

fromMs -- функция, возвращает тики таймера из миллисекунд.
local tdo = tm.fromMs( 1000 )

timeout -- функция, возвращает true, если истек таймаут
--параметры в тиках:
--1 - от тика
--2 - сколько тиков ждать
--3 - до тика (если не указан, то берется текущий тик таймера)
local cur = tm.curTick()
if tm.timeout( t, tdo, cur ) then --или так (cur-t) >= tdo
  --tdo или более тиков прошло от тика t до тика cur
end
if tm.timeout( t, tdo ) then --или так (tm.curTick()-t) >= tdo
  --tdo или более тиков прошло от тика t до текущего тика
end

-------------------------------------------------------------------
tmsSigs -- функция, возвращает таблицу функций для работы с сигналами, параметрами и т.д.
local tms = tmsSigs()

news -- возвращает массив id обновленных сигналов 
local ids = tms.news()
if ids == nil then
  return--нет новых
end
for k,v in pairs(ids) do
  local ds = tms.get( v )
  if ds ~= nil then
    log.DEBUG( function() return v," ",ds.VL," ",ds.ST," ",dt.frm( ds.DT ) end )
  end
end --for

get -- возвращает значение,дату,атрибуты сигнала.
--Параметр - id сигнала 
local ds = tms.get( 'id10' )
if ds ~= nil then
  log.DEBUG( function() return v," ",ds.VL," ",ds.ST," ",dt.frm( ds.DT ) end )
end

set -- передает tm-серверу значение,дату,атрибуты сигнала
--Параметр - id сигнала, таблица значение,дата,атрибуты
local ods = {}
ods.VL = 0 -- значение
ods.ST = st.aBAD --статус
ods.DT = dt.curDate() --дата
tms.set( 'id10', ods ) -- to tm_server

inIDs -- возвращает массив id сигналов входных в lua-скрипт
function tmsStart() --после старте 
  local ids = tms.inIDs()
  if ids == nil then
    return--нет входных сигналов
  end
  inSigs( ids ) -- все входные сигналы
end

outIDs -- возвращает массив id сигналов выходящих из lua-скрипта
ids = tms.outIDs()
if ids ~= nil then 
  for k,v in pairs(ids) do
    log.MES( 'id ', v ) 
  end
end

attrs -- возвращает таблицу констант атрибутов сигнала:
aUNC -– неопределенное положение двухпозиционного сигнала;
aCALC -– расчетный сигнал;
aUSE -– сигнал привязан к источнику;
aBLOCK -– значение сигнала заблокировано от изменения источником.
aOV -– переполнение;
aBL или aHAND -– значение ручное;
aSB или aREPL -– значение замещено;
aNT или aOLD -– значение устарело;
aIV или aBAD -– значение недостоверное (первичный сбор информации не выполнялся).

--Эти константы должны использоваться в битовых операциях с атрибутами сигнала
-- добавление, удаление, проверка атрибута:

local st = tms.attrs()
function isBad( ds )
  if ds.ST&st.aBAD ~= 0 then
    return true --bad
  end
  return false --not bad
end

pars -- возвращает таблицу параметров lua-скрипта
local pars = tms.pars()
log.MES( 'параметр rAddr=', pars.rAddr )

parseRet -- возвращает таблицу констант кодов возврата из функции tmsParse  
local ret = tms.parseRet()
ret.ERROR -- ошибка
ret.more -- продолжать прием
ret.NEXT -- перейти к следующему устройству.

-------------------------------------------------------------------
tmsUtils -- функция, возвращает таблицу вспомогательных функций
local utils = tmsUtils()

crc16 -- алгоритм CRC16 Modbus RTU
crc16ccitt -- алгоритм CRC16 CCITT 
--возвращают число
--параметры: строка и длина строки (необязательно)
local slen = s:len()
local crc1 = utils.crc16( s, slen - 2 ) --crc считается для s без двух последних байт
local crc2 = utils.crc16( s ) --crc считается для всей строки s


-------------------------------------------------------------------
tmsSvr -- функция, возвращает таблицу функций доступа к функциям и константам tm-сервера
local svr = tmsSvr()

tmsArh -- функция, возвращает имя архитектуры устройства
local arh = svr.tmsArh()
if arh == 'iRZR2' then
-- если на iRZR2
elseif arh == 'windows' then
-- если на windows
end

tmsVer -- функция, возвращает версию tm-сервера
local ver = svr.tmsVer()
if ver < 13652 then
  log.ERR( 'версия ', ver, 'не поддерживается' )
end

tmsDirTmp -- функция, возвращает временную папку
local tmp = svr.tmsDirTmp()

-------------------------------------------------------------------
tmsSys -- функция, возвращает таблицу функций доступа к системе
local sys = tmsSys()

isLinux --возвращает true, если работает на linux
if sys.isLinux() then
-- если на linux
end

isWin --возвращает true, если работает на windows
if sys.isWin() then
-- если на windows
end

errLast -- возвращает код последней ошибки
local errid = sys.errLast()

errStr --возвращает текстовое описание ошибки.
--Параметр код ошибки, если не указан, то берется код последней ошибки
log.ERR( sys.errStr( errid ) )
log.ERR( sys.errStr() )


-------------------------------------------------------------------
tmsRawData -- функция, возвращает таблицу функций для работы с raw data.
local raw = tmsRawData() 

rawRead -- чтение массива новых raw data
local RD = raw.rawRead() -- чтение массива новых raw data {id1:val1,id2:val2,...}
if RD == nil then
  return --нет новых
end
for id,val in pairs( RD ) do
  log.MES( "--> ", val.var )
  parseJson( val.data )
end
--см. tmsLoraJson.lua

rawWrite(id, rawdata) -- запись raw data - передача tm-серверу и далее подписчикам raw data
local jsonStr = '{"par1":1,"par2":2,"name":"name1"}'
raw.rawWrite( "parsjson", jsonStr )

-------------------------------------------------------------------
tmsNewJsonReader -- возвращает объект - парсер json (tmsJsonReaderClass)
local json = tmsNewJsonReader() -- json parser
--Реализация этого парсера похожа на JsonReader из Android.
--Нужно знать структуру json строки, последовательно её читать
--и извлекать данные.  
--Функции:
jsonOpen
jsonClose
beginArray
endArray
beginObject
endObject
hasNext
nextParam
nextValue
nextArrayValue
skipValue

--Пример из tmsLoraJson.lua показывает логику работы с парсером. 

function parseJson( s )
  log.MES( "--> ", s )
  curDT = dt.curDate()
  json:jsonOpen( s ) --строку в json parser
--чтение массива объектов [{"p1":"v1","p1":"v1"},{..},..]  
  json:beginArray() --[
  while json:hasNext() do --,
--чтение объекта  
    json:beginObject() --{
    local par, val, desc, pid, date = curDT
    while json:hasNext() do --,
      par = json:nextParam() --"PARAM":
      if par == 'value' then
        val = json:nextValue() --:"VALUE"
--        log.DEBUG(par,": ",val)
      elseif par == 'description' then
        desc = json:nextValue()
--        log.DEBUG(par,": ",desc)
      elseif par == 'param_id' then
        pid = tonumber( json:nextValue() )
--        log.DEBUG(par,": ",pid)
      elseif par == 'data_timestamp' then
        date = tonumber( json:nextValue() )
        date = date * 1000 --ms
--        log.DEBUG(par,": ",date)
      else
        json:skipValue()
      end
    end
    json:endObject() --}
-- объект прочитан - выполнить обработку    
    log.DEBUG(pid,": ",val," - ", desc)
    if pid == -1 then --raw data
      raw.rawWrite( "loraOut", val )
      parseRawData( val )
    else --параметры
      sigSend( 'pid'..pid, val, date )
    end
  end
  json:endArray() --]
  json:jsonClose() --удалить строку из json parser
end --parseJson

-------------------------------------------------------------------
tmsNewSQLiteDB -- возвращает объект SQLite база данных (tmsSQLiteDBClass)
local db = tmsNewSQLiteDB() --SQLite БД

--Функции:
dbOpen -- открывает файл БД
db:dbOpen( '/tmp/db1.sqlite' )

dbClose -- закрывает файл БД
db:dbClose()

dbExec -- выполняет SQL оператор
db:dbExec( 'BEGIN' )
db:dbExec( 'DELETE FROM tPAR WHERE fID="'..id..'"' )
db:dbExec( 'INSERT INTO tPAR VALUES("'..id..'",'..vl..')' )
db:dbExec( 'END' )

errCatch -- управление перехватом ошибок, поумолчанию, если вызов функции генерирует ошибку,
-- ошибка перехватывается, выполнение функции завершает и ошибка пишется в лог Lua-скрипта.
-- Если нужно ошибку обработать в lua-скрипте, то нужно оключить перхват ошибок.
errGet -- возвращает последнюю ошибку, если перхват был отключен
db:errCatch( false ) --не перехватывать ошибки
db:dbExec( sql )
local eNum, eStr = db:errGet() --получить ошибку
if eNum == 0 then
  --ошибок нет
else
  log.ERR( eStr )
end
db:errCatch( true ) --перехватывать ошибки

dbNewSelect -- возвращает SELECT-объект для выполнения SELECT запросов (tmsSQLiteSelectClass).
-- Обычно достаточно одного SELECT-объекта, но можно создать несколько для одной БД.
-- Не рекомендую создавать локальные SELECT-объекты в функциях.
-- Рекомендую создавать нужное количество SELECT-объектов запросов в начале lua-скрипта
-- и затем их использовать.
local sel = db:dbNewSelect() --для SELECT запросов к БД

selOpen -- окрывает SELECT-запрос
selClose -- закрывает SELECT-запрос
selNext -- переход на следующую строку запроса 
selEnd -- возвращает true, если нет достигнут конец запроса
selVal -- возвращает значение столбца текущей строки по номеру или имени
sel:selOpen( 'SELECT * FROM tPAR )
local id, vl
while not sel:selEnd() do
  id = sel:selVal( 'fID' )
  vl = sel:selVal( 'fVL' )
  log.MES( id..' = '..vl )
  sel:selNext()
end
sel:selClose()

selColCount -- возвращает количество столбцов
selColName -- возвращает имя столбца по номеру
selValType -- возвращает тип столбца по номеру или имени
selTypes -- возвращает таблицу констант типов столбцов
SQLITE_INTEGER
SQLITE_FLOAT
SQLITE_TEXT
SQLITE_BLOB
SQLITE_NULL

local tps = sel:selTypes()
if sel:selValType( 'name' ) == tps.SQLITE_NULL then
--NULL
end

errCatch и errGet -- управление перехватом ошибок, как и для объекта БД (см выше )

-------------------------------------------------------------------
tmsFile -- функция, возвращает таблицу функций для доступа к файлам
local file = tmsFile() --файлы

fileWrite -- запись строки в файл, новый файл создается, старый удаляется 
if not file.fileWrite( filename, s ) then
  log.ERR( 'do not write file '..filename )
end

fileAppend -- добавление строки в файл, если файла нет, то новый файл создается
if not file.fileAppend( filename, s ) then
  log.ERR( 'do not append file '..filename )
end

fileRead -- чтение файла в строку
--параметры:
  -- filename - имя файла
  -- size - сколько байт максимально читать (поумолчанию 1МБ)
  -- pos - позиция от начала файла (поумолчанию 0)
file.fileRead( filename, size, pos )
  
local s = file.fileRead( filename )
if s == nil then
 log.ERR( 'do not read file '..filename )
end

fileDel -- удаление файла
if file.fileDel( filename ) ~= 0 then
 log.ERR( 'do not del file '..filename )
end

fileRename -- переименование файла
if file.fileRename( old_filename, new_filename ) ~= 0 then
  log.ERR( 'do not rename file '..old_filename..' to '..new_filename )
end

fileSize -- размер файла
local sz = file.fileSize( filename )
if sz < 0 then
  log.ERR( 'do not get size of file '..filename )
end

fileExists -- проверка существования файла
if not file.fileExists( filename ) then
  log.ERR( 'do not exists file '..filename )
end

dirExists -- проверка существования папки
if not file.dirExists( dirname ) then
  log.ERR( 'do not exists dir '..dirname )
end

dirForce -- рекурсивное создание папки
if not file.dirForce( dirname ) then
  log.ERR( 'do not create dir '..dirname )
end


fileConst -- константы для работы с файлами
local fc = file.fileConst()
 ppDRIVE -- диск (d:)
 ppDIR -- папка
 ppNAME -- имя 
 ppEXT -- расширение
 ppDRIVE_DIR_NAME -- диск + папка + имя
 ppNAME_EXT -- имя + расширение
 ppDRIVE_DIR -- диск + папка
 dirSep --символ - разделитель папок 
 -- / - linux
 -- \ - windows
 
filePath -- извлечение частей пути файла, см. fileConst
local fn = 'c:\\tm\\db\\pars.sqlite'
local s1 = file.filePath( fn, fc.ppNAME ) --pars
local s2 = file.filePath( fn, fc.ppNAME_EXT ) --pars.sqlite
local s3 = file.filePath( fn, fc.ppDRIVE_DIR ) --c:\tm\db\

fileStat --возвращает таблицу с информацией о файле (struct stat):
dev     -- ID of device containing file
ino     -- inode number
mode    -- protection
nlink   -- number of hard links
uid     -- user ID of owner
gid     -- group ID of owner
rdev    -- device ID (if special file)
size    -- total size, in bytes
atime   -- time of last access
mtime   -- time of last modification
ctime   -- time of last status change

fileChmod --устанавливает права для файла (для linux)
fileChmodConst --возвращает константы прав для файла (для linux)
SUID -- S_ISUID (04000) set-user-ID (set process effective user ID on execve(2))
SGID -- S_ISGID (02000) set-group-ID (set process effective group ID on execve(2);
              --mandatory locking, as described in fcntl(2); take a new
              --file's group from parent directory, as described in
              --chown(2) and mkdir(2))
VTX -- S_ISVTX (01000) sticky bit (restricted deletion flag, as described in unlink(2))
ur -- S_IRUSR (00400) read by owner
uw -- S_IWUSR (00200) write by owner
ux -- S_IXUSR (00100) execute/search by owner ("search" applies for directories,
              --and means that entries within the directory can be accessed)
gr -- S_IRGRP (00040) read by group
gw -- S_IWGRP (00020) write by group
gx -- S_IXGRP (00010) execute/search by group
or -- S_IROTH (00004) read by others
ow -- S_IWOTH (00002) write by others
ox -- S_IXOTH (00001) execute/search by others

local cmdfn = '/tmp/tm_cpps/temp/testcmd.sh'
local fs = file.fileStat( cmdfn )
if fs == nil then
  log.ERR( cmdfn, ' не удалось прочитать stat', sys.errStr() )
else
  log.MES( cmdfn
  , '\ntime of last access        ', dt.frm( fs.atime*1000 )
  , '\ntime of last modification  ', dt.frm( fs.mtime*1000 )
  , '\ntime of last status change ', dt.frm( fs.ctime*1000 ) )
end
if sys.isLinux() then -- установить права на запуск
  local cc = file.fileChmodConst()
  local m = cc.ur | cc.uw | cc.ux --чтение+запись+запуск для владельца
  if fs ~= nil then
    m = m | fs.mode -- + права, которые были
  end
  if not file.fileChmod( cmdfn, m ) then
    log.ERR( cmdfn, ' не удалось установить права', sys.errStr() )
  end
end

-------------------------------------------------------------------
tmsNewProcess -- функция,  возвращает объект для запуска и управления процессом
local pr = tmsNewProcess()

prStart --запуск процесса
--Процесс запускается, а объект pr хранит информацию о процессе.
--для windows
pr:prStart( 'ping 127.0.0.1' )
pr:prStart( 'ping', '127.0.0.1' )
pr:prStart( {[1]='ping', [2]='127.0.0.1'} )
--для Linux - полный путь к процессу обязателен !!!
pr:prStart( '/bin/ping', '127.0.0.1', '-c3' )
pr:prStart( {[1]='/bin/ping', [2]='127.0.0.1', [3]='-c3'} )

--Перенаправление вывода команды в файл выполнять в bat или sh файле.
--пример bat - для windows
local cmd = 'chcp 65001\r\n' --результат в UTF-8
cmd = cmd..'ping 127.0.0.1 > c:\\1\\ping.txt\r\n'
file.fileWrite( 'C:\\1\\ping.bat', cmd )
pr:prStart( 'C:\\1\\ping.bat' )
pr:prWait( 10000 )--ждать завершения
local res = file.fileRead( 'c:\\1\\ping.txt' )

--пример sh - для Linux
local cmd = '#!/bin/sh\n'
cmd = cmd..'ls / > /tmp/lsres\n'
file.fileWrite( '/tmp/lscmd', cmd )
pr:prStart( '/bin/chmod', '+x', '/tmp/lscmd' ) --права на запуск
pr:prWait( 5000 )--ждать завершения
pr:prReset()
pr:prStart( '/tmp/lscmd' )
pr:prWait( 5000 )--ждать завершения
local res = file.fileRead( '/tmp/lsres' )

prReset -- сброс иинформации о процессе,
--что бы использовать объект pr для управления другим процессом.
--Рекомендую не создавать локальных объектов pr,
--а повторно использовать объект pr после сброса.
pr:prReset()

prStop --остановка процесса
pr:prStop()

prKill --убить процесс, если не останавливается или завис
pr:prKill()

prWait --ждать завершения процесса, параметр таймаут максимального ожидания.
--Выполнение lua-скрипта останаливается до завершения процесса или до окончания таймаута.
--Возвращает:
-- -1 - ошибка
--  0 - процесс завершился
--  1 - истек таймаут
local res = pr:prWait( 5000 ) --ждать 5 секунд
if res == 0 then
  log.MES( 'завершился' )
elseif res == 1 then
  log.MES( 'пока выполняется' )
else
  log.ERR( 'ошибка' )
end

prExitCode --возвращает код завершения процесса, код устанавливает процесс.
pr:prExitCode()

prTest -- возвращает true, если процесс выполняется.
prID -- возвращает id процесса
if pr:prID() > 0 then --есть процесс в pr объекте
  if pr:prTest() then --процесс выполняется
    log.MES( 'выполняется' )
  end
end

errCatch errGet -- управление перехватом ошибок, как и для объекта БД (см выше )
-------------------------------------------------------------------




